% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/string.R
\name{is_ipv6}
\alias{is_ipv6}
\title{Check if an argument is an IPv6 address string}
\usage{
is_ipv6(x)
}
\arguments{
\item{x}{(\code{any})\cr
Object to check.}
}
\value{
\code{TRUE} if \code{x} is a valid IPv6 address string, \code{FALSE} otherwise.
}
\description{
Validates full, compressed (\code{::}) and IPv4-embedded (\verb{::ffff:192.168.1.1}) forms.
Zone IDs (e.g. \verb{\%eth0}) are not supported.
}
\examples{
is_ipv6("2001:0db8:85a3:0000:0000:8a2e:0370:7334")
is_ipv6("::1")
}
