% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/string.R
\name{is_semver}
\alias{is_semver}
\title{Check if an argument is a semantic versioning string}
\usage{
is_semver(x)
}
\arguments{
\item{x}{(\code{any})\cr
Object to check.}
}
\value{
\code{TRUE} if \code{x} is a valid semver string, \code{FALSE} otherwise.
}
\description{
Validates semantic versioning 2.0.0 format, including optional pre-release and build metadata.
Uses the official recommended regex from \url{https://semver.org/}.
}
\examples{
is_semver("1.0.0")
}
\references{
\url{https://semver.org/}
}
