% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/string.R
\name{is_color_hex}
\alias{is_color_hex}
\title{Check if an argument is a hex color string}
\usage{
is_color_hex(x)
}
\arguments{
\item{x}{(\code{any})\cr
Object to check.}
}
\value{
\code{TRUE} if \code{x} is a valid hex color string, \code{FALSE} otherwise.
}
\description{
Validates hex color codes in \verb{#RGB}, \verb{#RGBA}, \verb{#RRGGBB}, or \verb{#RRGGBBAA} format.
}
\examples{
is_color_hex("#FF5733")
is_color_hex("#fff")
}
\references{
\url{https://www.w3.org/TR/css-color-4/#hex-notation}
}
